@@ -23,6 +23,8 @@
 #include "libedataserver/e-data-server-util.h"
 #include "e-book-backend-sexp.h"
 
+G_DEFINE_TYPE (EBookBackendSExp, e_book_backend_sexp, G_TYPE_OBJECT)
+
 static GObjectClass *parent_class;
 
 typedef struct _SearchContext SearchContext;
@@ -382,7 +384,7 @@ entry_compare(SearchContext *ctx, struct _ESExp *f,
 		}
 	}
 	r = e_sexp_result_new(f, ESEXP_RES_BOOL);
-	r->value.bool = truth;
+	r->value.boolean = truth;
 
 	return r;
 }
@@ -742,7 +744,7 @@ func_exists(struct _ESExp *f, gint argc, struct _ESExpResult **argv, gpointer da
 		}
 	}
 	r = e_sexp_result_new(f, ESEXP_RES_BOOL);
-	r->value.bool = truth;
+	r->value.boolean = truth;
 
 	return r;
 }
@@ -774,7 +776,7 @@ func_exists_vcard(struct _ESExp *f, gint argc, struct _ESExpResult **argv, gpoin
 	}
 
 	r = e_sexp_result_new(f, ESEXP_RES_BOOL);
-	r->value.bool = truth;
+	r->value.boolean = truth;
 
 	return r;
 }
@@ -801,7 +803,7 @@ static struct {
  *
  * Checks if @contact matches @sexp.
  *
- * Return value: %TRUE if the contact matches, %FALSE otherwise.
+ * Returns: %TRUE if the contact matches, %FALSE otherwise.
  **/
 gboolean
 e_book_backend_sexp_match_contact (EBookBackendSExp *sexp, EContact *contact)
@@ -818,7 +820,7 @@ e_book_backend_sexp_match_contact (EBookBackendSExp *sexp, EContact *contact)
 
 	r = e_sexp_eval(sexp->priv->search_sexp);
 
-	retval = (r && r->type == ESEXP_RES_BOOL && r->value.bool);
+	retval = (r && r->type == ESEXP_RES_BOOL && r->value.boolean);
 
 	g_object_unref(sexp->priv->search_context->contact);
 
@@ -834,7 +836,7 @@ e_book_backend_sexp_match_contact (EBookBackendSExp *sexp, EContact *contact)
  *
  * Checks if @vcard matches @sexp.
  *
- * Return value: %TRUE if the VCard matches, %FALSE otherwise.
+ * Returns: %TRUE if the VCard matches, %FALSE otherwise.
  **/
 gboolean
 e_book_backend_sexp_match_vcard (EBookBackendSExp *sexp, const gchar *vcard)
@@ -859,7 +861,7 @@ e_book_backend_sexp_match_vcard (EBookBackendSExp *sexp, const gchar *vcard)
  *
  * Creates a new #EBookBackendSExp from @text.
  *
- * Return value: A new #EBookBackendSExp.
+ * Returns: A new #EBookBackendSExp.
  **/
 EBookBackendSExp *
 e_book_backend_sexp_new (const gchar *text)
@@ -931,30 +933,3 @@ e_book_backend_sexp_init (EBookBackendSExp *sexp)
 	sexp->priv = priv;
 	priv->search_context = g_new (SearchContext, 1);
 }
-
-/**
- * e_book_backend_sexp_get_type:
- */
-GType
-e_book_backend_sexp_get_type (void)
-{
-	static GType type = 0;
-
-	if (!type) {
-		GTypeInfo info = {
-			sizeof (EBookBackendSExpClass),
-			NULL, /* base_class_init */
-			NULL, /* base_class_finalize */
-			(GClassInitFunc)  e_book_backend_sexp_class_init,
-			NULL, /* class_finalize */
-			NULL, /* class_data */
-			sizeof (EBookBackendSExp),
-			0,    /* n_preallocs */
-			(GInstanceInitFunc) e_book_backend_sexp_init
-		};
-
-		type = g_type_register_static (G_TYPE_OBJECT, "EBookBackendSExp", &info, 0);
-	}
-
-	return type;
-}
