@@ -2,7 +2,7 @@
  *
  * Author: Michael Zucchi <notzed@ximian.com>
  *
- * Copyright 2004 Novell Inc. (www.novell.com)
+ * Copyright (C) 1999-2008 Novell, Inc. (www.novell.com)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 2 of the GNU Lesser General Public
@@ -19,8 +19,12 @@
  * USA
  */
 
+#if !defined (__CAMEL_H_INSIDE__) && !defined (CAMEL_COMPILATION)
+#error "Only <camel/camel.h> can be included directly."
+#endif
+
 #ifndef CAMEL_DEBUG_H
-#define CAMEL_DEBUG_H 1
+#define CAMEL_DEBUG_H
 
 #include <glib.h>
 
@@ -31,14 +35,11 @@
 G_BEGIN_DECLS
 
 void camel_debug_init(void);
-gboolean camel_debug(const char *mode);
+gboolean camel_debug(const gchar *mode);
 
-gboolean camel_debug_start(const char *mode);
+gboolean camel_debug_start(const gchar *mode);
 void camel_debug_end(void);
 
-/* This interface is deprecated */
-extern int camel_verbose_debug;
-
 G_END_DECLS
 
 #endif /* CAMEL_DEBUG_H */
