@@ -1,15 +1,15 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /* camel-execpetion.h : exception utils */
 
-/* 
+/*
  *
- * Author : 
+ * Author :
  *  Bertrand Guiheneuf <bertrand@helixcode.com>
  *
- * Copyright 1999, 2000 Ximian, Inc. (www.ximian.com)
+ * Copyright (C) 1999-2008 Novell, Inc. (www.novell.com)
  *
- * This program is free software; you can redistribute it and/or 
- * modify it under the terms of version 2 of the GNU Lesser General Public 
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of version 2 of the GNU Lesser General Public
  * License as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -19,19 +19,23 @@
  *
  * You should have received a copy of the GNU Lesser General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  * USA
  */
 
-
+#if !defined (__CAMEL_H_INSIDE__) && !defined (CAMEL_COMPILATION)
+#error "Only <camel/camel.h> can be included directly."
+#endif
 
 #ifndef CAMEL_EXCEPTION_H
-#define CAMEL_EXCEPTION_H 1
+#define CAMEL_EXCEPTION_H
 
-#include <camel/camel-types.h>
+#include <glib.h>
 
 G_BEGIN_DECLS
 
+typedef struct _CamelException CamelException;
+
 typedef enum {
 #include "camel-exception-list.def"
 
@@ -40,7 +44,7 @@ typedef enum {
 struct _CamelException {
 	/* do not access the fields directly */
 	ExceptionId id;
-	char *desc;
+	gchar *desc;
 };
 
 #define CAMEL_EXCEPTION_INITIALISER { 0, NULL }
@@ -50,25 +54,23 @@ CamelException *          camel_exception_new           (void);
 void                      camel_exception_free          (CamelException *ex);
 void                      camel_exception_init          (CamelException *ex);
 
-
 /* exception content manipulation */
 void                      camel_exception_clear         (CamelException *ex);
 void                      camel_exception_set           (CamelException *ex,
 							 ExceptionId id,
-							 const char *desc);
+							 const gchar *desc);
 void                      camel_exception_setv          (CamelException *ex,
 							 ExceptionId id,
-							 const char *format,  
+							 const gchar *format,
 							 ...);
 
 /* exception content transfer */
 void                      camel_exception_xfer          (CamelException *ex_dst,
 							 CamelException *ex_src);
 
-
 /* exception content retrieval */
 ExceptionId               camel_exception_get_id        (CamelException *ex);
-const char *             camel_exception_get_description (CamelException *ex);
+const gchar *             camel_exception_get_description (CamelException *ex);
 
 #define camel_exception_is_set(ex) (camel_exception_get_id (ex) != CAMEL_EXCEPTION_NONE)
 
