@@ -24,7 +24,6 @@
 #include <config.h>
 #endif
 
-#include <glib.h>
 #include <glib/gi18n-lib.h>
 
 #include "camel-folder.h"
@@ -32,68 +31,43 @@
 #include "camel-offline-store.h"
 #include "camel-session.h"
 
-static void camel_offline_store_class_init (CamelOfflineStoreClass *klass);
-static void camel_offline_store_init (CamelOfflineStore *store, CamelOfflineStoreClass *klass);
-static void camel_offline_store_finalize (CamelObject *object);
+G_DEFINE_TYPE (CamelOfflineStore, camel_offline_store, CAMEL_TYPE_STORE)
 
-static void offline_store_construct (CamelService *service, CamelSession *session,
-				     CamelProvider *provider, CamelURL *url,
-				     CamelException *ex);
-
-static CamelStoreClass *parent_class = NULL;
-
-CamelType
-camel_offline_store_get_type (void)
-{
-	static CamelType type = NULL;
-
-	if (!type) {
-		type = camel_type_register (CAMEL_STORE_TYPE,
-					    "CamelOfflineStore",
-					    sizeof (CamelOfflineStore),
-					    sizeof (CamelOfflineStoreClass),
-					    (CamelObjectClassInitFunc) camel_offline_store_class_init,
-					    NULL,
-					    (CamelObjectInitFunc) camel_offline_store_init,
-					    (CamelObjectFinalizeFunc) camel_offline_store_finalize);
-	}
-
-	return type;
-}
-
-static void
-camel_offline_store_class_init (CamelOfflineStoreClass *klass)
-{
-	parent_class = (CamelStoreClass *) camel_type_get_global_classfuncs (CAMEL_STORE_TYPE);
-
-	((CamelServiceClass *) klass)->construct = offline_store_construct;
-}
-
-static void
-camel_offline_store_init (CamelOfflineStore *store, CamelOfflineStoreClass *klass)
-{
-	store->state = CAMEL_OFFLINE_STORE_NETWORK_AVAIL;
-}
-
-static void
-camel_offline_store_finalize (CamelObject *object)
-{
-	;
-}
-
-static void
-offline_store_construct (CamelService *service, CamelSession *session,
-			  CamelProvider *provider, CamelURL *url,
-			  CamelException *ex)
+static gboolean
+offline_store_construct (CamelService *service,
+                         CamelSession *session,
+                         CamelProvider *provider,
+                         CamelURL *url,
+                         CamelException *ex)
 {
 	CamelOfflineStore *store = CAMEL_OFFLINE_STORE (service);
+	CamelServiceClass *service_class;
 
-	CAMEL_SERVICE_CLASS (parent_class)->construct (service, session, provider, url, ex);
-	if (camel_exception_is_set (ex))
-		return;
+	/* Chain up to parent's construct() method. */
+	service_class = CAMEL_SERVICE_CLASS (camel_offline_store_parent_class);
+	if (!service_class->construct (service, session, provider, url, ex))
+		return FALSE;
 
 	store->state = camel_session_is_online (session) ?
-		CAMEL_OFFLINE_STORE_NETWORK_AVAIL : CAMEL_OFFLINE_STORE_NETWORK_UNAVAIL;
+		CAMEL_OFFLINE_STORE_NETWORK_AVAIL :
+		CAMEL_OFFLINE_STORE_NETWORK_UNAVAIL;
+
+	return TRUE;
+}
+
+static void
+camel_offline_store_class_init (CamelOfflineStoreClass *class)
+{
+	CamelServiceClass *service_class;
+
+	service_class = CAMEL_SERVICE_CLASS (class);
+	service_class->construct = offline_store_construct;
+}
+
+static void
+camel_offline_store_init (CamelOfflineStore *store)
+{
+	store->state = CAMEL_OFFLINE_STORE_NETWORK_AVAIL;
 }
 
 /**
@@ -103,9 +77,12 @@ offline_store_construct (CamelService *service, CamelSession *session,
  *
  * Return the network state either #CAMEL_OFFLINE_STORE_NETWORK_AVAIL
  * or #CAMEL_OFFLINE_STORE_NETWORK_UNAVAIL.
+ *
+ * Since: 2.24
  **/
 gint
-camel_offline_store_get_network_state (CamelOfflineStore *store, CamelException *ex)
+camel_offline_store_get_network_state (CamelOfflineStore *store,
+                                       CamelException *ex)
 {
 	return store->state;
 }
@@ -119,15 +96,17 @@ camel_offline_store_get_network_state (CamelOfflineStore *store, CamelException
  * Set the network state to either #CAMEL_OFFLINE_STORE_NETWORK_AVAIL
  * or #CAMEL_OFFLINE_STORE_NETWORK_UNAVAIL.
  **/
-void
-camel_offline_store_set_network_state (CamelOfflineStore *store, gint state, CamelException *ex)
+gboolean
+camel_offline_store_set_network_state (CamelOfflineStore *store,
+                                       gint state,
+                                       CamelException *ex)
 {
 	CamelException lex;
 	CamelService *service = CAMEL_SERVICE (store);
 	gboolean network_state = camel_session_get_network_state (service->session);
 
 	if (store->state == state)
-		return;
+		return TRUE;
 
 	camel_exception_init (&lex);
 	if (store->state == CAMEL_OFFLINE_STORE_NETWORK_AVAIL) {
@@ -144,13 +123,13 @@ camel_offline_store_set_network_state (CamelOfflineStore *store, gint state, Cam
 				for (i = 0; i < folders->len; i++) {
 					folder = folders->pdata[i];
 
-					if (CAMEL_CHECK_TYPE (folder, CAMEL_OFFLINE_FOLDER_TYPE)
-					    && (sync || ((CamelOfflineFolder *) folder)->sync_offline)) {
+					if (G_TYPE_CHECK_INSTANCE_TYPE (folder, CAMEL_TYPE_OFFLINE_FOLDER)
+					    && (sync || camel_offline_folder_get_offline_sync (CAMEL_OFFLINE_FOLDER (folder)))) {
 						camel_offline_folder_downsync ((CamelOfflineFolder *) folder, NULL, &lex);
 						camel_exception_clear (&lex);
 					}
 
-					camel_object_unref (folder);
+					g_object_unref (folder);
 				}
 
 				g_ptr_array_free (folders, TRUE);
@@ -161,21 +140,29 @@ camel_offline_store_set_network_state (CamelOfflineStore *store, gint state, Cam
 		}
 
 		if (!camel_service_disconnect (CAMEL_SERVICE (store), network_state, ex))
-			return;
+			return FALSE;
 	} else {
 		store->state = state;
 		/* network unavailable -> network available */
 		if (!camel_service_connect (CAMEL_SERVICE (store), ex)) {
 			store->state = CAMEL_OFFLINE_STORE_NETWORK_UNAVAIL;
-			return;
+			return FALSE;
 		}
 	}
 
 	store->state = state;
+
+	return TRUE;
 }
 
-void
-camel_offline_store_prepare_for_offline (CamelOfflineStore *store, CamelException *ex)
+/**
+ * camel_offline_store_prepare_for_offline:
+ *
+ * Since: 2.22
+ **/
+gboolean
+camel_offline_store_prepare_for_offline (CamelOfflineStore *store,
+                                         CamelException *ex)
 {
 	CamelException lex;
 	CamelService *service = CAMEL_SERVICE (store);
@@ -195,12 +182,12 @@ camel_offline_store_prepare_for_offline (CamelOfflineStore *store, CamelExceptio
 				for (i = 0; i < folders->len; i++) {
 					folder = folders->pdata[i];
 
-					if (CAMEL_CHECK_TYPE (folder, CAMEL_OFFLINE_FOLDER_TYPE)
-					    && (sync || ((CamelOfflineFolder *) folder)->sync_offline)) {
+					if (G_TYPE_CHECK_INSTANCE_TYPE (folder, CAMEL_TYPE_OFFLINE_FOLDER)
+					    && (sync || camel_offline_folder_get_offline_sync (CAMEL_OFFLINE_FOLDER (folder)))) {
 						camel_offline_folder_downsync ((CamelOfflineFolder *) folder, NULL, &lex);
 						camel_exception_clear (&lex);
 					}
-					camel_object_unref (folder);
+					g_object_unref (folder);
 				}
 				g_ptr_array_free (folders, TRUE);
 			}
@@ -208,6 +195,7 @@ camel_offline_store_prepare_for_offline (CamelOfflineStore *store, CamelExceptio
 
 		camel_store_sync (CAMEL_STORE (store), FALSE, &lex);
 		camel_exception_clear (&lex);
-
 	}
+
+	return TRUE;
 }
